/*
* subalign.c -- print a range of columns from a multiple alignment,
* as indexed by positions in the reference sequence.  (a.k.a. slice.c)
*
* $Revision: 1.7 $$Date: 2004/09/10 19:59:27 $
*/

#include "util.h"
#include "read_malign.h"

#define SYNTAX1  "multipipmaker-text-alignment from to [D=0-or-1]"
#define SYNTAX2  "multipipmaker-text-alignment interval-file [D=0-or-1]"

enum { BUFSIZE = 1000 };

static bool parse_line(char *buf, int *from, int *to);
static void print_range(char **A, int nrow, int ncol, char **seq_name, int from, int to, bool dots);

int main(int argc, char **argv)
{
	char **A, **seq_name, *posfile = NULL, buf[BUFSIZE];
	int  nrow, ncol, from, to;
	bool dots;
	FILE *fp;

	argv0 = "subalign";
	if (argc < 3)
		fatalf("usage:\n\t%s %s\n\t%s %s", argv0, SYNTAX1, argv0, SYNTAX2);
	A = read_malign(argv[1], &nrow, &ncol, &seq_name);

	if (strspn(argv[2], "0123456789") == strlen(argv[2])) {    /* 2nd arg is a number */
		if (argc < 4 || argc > 5)
			fatalf("usage:\n\t%s %s\n\t%s %s", argv0, SYNTAX1, argv0, SYNTAX2);
		from = atoi(argv[2]);
		to = atoi(argv[3]);
		dots = (argc == 5 && same_string(argv[4], "D=1"));
		print_range(A, nrow, ncol, seq_name, from, to, dots);
	}
	else {    /* 2nd arg is an interval file */
		if (argc < 3 || argc > 4)
			fatalf("usage:\n\t%s %s\n\t%s %s", argv0, SYNTAX1, argv0, SYNTAX2);
		posfile = argv[2];
		dots = (argc == 4 && same_string(argv[3], "D=1"));

		fp = ckopen(posfile, "r");
		while (fgets(buf, BUFSIZE, fp)) {
			if (strlen(buf) == BUFSIZE-1)    /* guarantees \n, 1 empty char */
				fatalf("line too long in \"%s\":\n%s", posfile, buf);
			if (parse_line(buf, &from, &to))
				print_range(A, nrow, ncol, seq_name, from, to, dots);
		}
		fclose (fp);
	}
	return 0;
}

static bool parse_line(char *buf, int *from, int *to)
{
	char *b = skip_ws(buf);
	if (!*b || *b == '#')    /* blank line or comment */
		return 0;
	if (sscanf(b, "%d%*[ \t]%d%*[ \t\n]", from, to) == 2)    /* numbers first */
		return 1;
	if (sscanf(b, "%*[><|+]%d%*[ \t]%d%*[ \t\n]", from, to) == 2)    /* exons file */
		return 1;
	while (*b) {
		while (*b && !isspace(*b))    /* look for leading whitespace */
			b++;
		if (sscanf(b, "%*[ \t]%d%*[ \t]%d%*[ \t\n]", from, to) == 2)
			return 1;
		b = skip_ws(b);
	}
	fatalf("can't find interval in line:\n%s", buf);
	return 0;    /* pacify lint */
}

static void print_range(char **A, int nrow, int ncol, char **seq_name, int from, int to, bool dots)
{
	int  f, t, i, j, k;
	char x;

	/* check endpoints */
	if (from <= 0 || to <= 0)
		fatalf("invalid sequence positions: %d %d", from, to);
	if (from > to)
		fatalf("endpoints given in wrong order: %d %d", from, to);

	/* convert given positions in the first sequence to column numbers */
	f = t = -1;
	for (j = k = 0; j < ncol; ++j) {
		if (isalpha(A[0][j])) {
			if (++k == from)
				f = j;
			if (k == to) {
				t = j;
				break;
			}
		}
	}
	if (j >= ncol)
		fatalf("positions too large for alignment: %d %d", from, to);

	/* print columns in the given range */
	printf("%d - %d\n", from, to);
	for (i = 0; i < nrow; ++i) {
		for (j = f; j <= t; ++j) {
			x = A[i][j];
			if (dots && i > 0 && x == A[0][j])
				x = '.';
			putchar(x);
		}
		printf("  %s\n", seq_name[i]);
	}
	putchar('\n');

	return;
}

